#define _WIN32_WINNT 0x0500
#define NO_WINTERN_PEB
#define WINBASE_DECLARE_GET_MODULE_HANDLE_EX

#include <windows.h>
#define PPEB PVOID
#include <winternl.h>
#undef PPEB

#include <stdio.h>

typedef LONG NTSTATUS;
#define NT_SUCCESS(Status) ((NTSTATUS)(Status) >= 0)

typedef enum _DDK_PROCESSINFOCLASS {
    DDK_ProcessBasicInformation,
    ProcessQuotaLimits,
    ProcessIoCounters,
    ProcessVmCounters,
    ProcessTimes,
    ProcessBasePriority,
    ProcessRaisePriority,
    ProcessDebugPort,
    ProcessExceptionPort,
    ProcessAccessToken,
    ProcessLdtInformation,
    ProcessLdtSize,
    ProcessDefaultHardErrorMode,
    ProcessIoPortHandlers,          // Note: this is kernel mode only
    ProcessPooledUsageAndLimits,
    ProcessWorkingSetWatch,
    ProcessUserModeIOPL,
    ProcessEnableAlignmentFaultFixup,
    ProcessPriorityClass,
    ProcessWx86Information,
    ProcessHandleCount,
    ProcessAffinityMask,
    ProcessPriorityBoost,
    ProcessDeviceMap,
    ProcessSessionInformation,
    ProcessForegroundInformation,
    DDK_ProcessWow64Information,
    MaxProcessInfoClass
    } DDK_PROCESSINFOCLASS;

typedef struct _LDR_MODULE {
	LIST_ENTRY InLoadOrderModuleList;
	LIST_ENTRY InMemoryOrderModuleList;
	LIST_ENTRY InInitializationOrderModuleList;
	PVOID BaseAddress;
	PVOID EntryPoint;
	ULONG SizeOfImage;
	UNICODE_STRING FullDllName;
	UNICODE_STRING BaseDllName;
	ULONG Flags;
	SHORT LoadCount;
	SHORT TlsIndex;
	LIST_ENTRY HashTableEntry;
	ULONG TimeDateStamp;
} LDR_MODULE, *PLDR_MODULE;

typedef struct _PEB_LDR_DATA {
	ULONG Length;
	BOOLEAN Initialized;
	PVOID SsHandle;
	LIST_ENTRY InLoadOrderModuleList;
	LIST_ENTRY InMemoryOrderModuleList;
	LIST_ENTRY InInitializationOrderModuleList;
} PEB_LDR_DATA, *PPEB_LDR_DATA;

typedef struct _PEB {
	BOOLEAN InheritedAddressSpace;
	BOOLEAN ReadImageFileExecOptions;
	BOOLEAN BeingDebugged;
	BOOLEAN Spare; 
	HANDLE Mutant;
	PVOID ImageBaseAddress;
	PPEB_LDR_DATA LoaderData;
	//And more but we don't care...
} PEB, *PPEB;

typedef struct _CLIENT_ID {
    HANDLE UniqueProcess;
    HANDLE UniqueThread;
} CLIENT_ID;
typedef CLIENT_ID *PCLIENT_ID;

typedef struct _THREAD_BASIC_INFORMATION {
	NTSTATUS ExitStatus;
	PVOID TebBaseAddress;
	CLIENT_ID ClientId;
	KAFFINITY AffinityMask;
	//KPRIORITY Priority;
	//KPRIORITY BasePriority;
} THREAD_BASIC_INFORMATION, *PTHREAD_BASIC_INFORMATION;

#define ThreadBasicInformation 0

OSVERSIONINFOEXA versionInfoA;
OSVERSIONINFOEXW versionInfoW;

BOOL GetModuleOriginal;
HMODULE thisModule;
HMODULE hookedModule;

GEOID geoRegion;
GEOID geoNation;

WCHAR systemDirectoryW[MAX_PATH];
CHAR  systemDirectoryA[MAX_PATH];

//GetGeoInfo
CHAR geoISO2A[3];
CHAR geoISO3A[4];

BOOL RewriteExportTable = FALSE;

PLDR_MODULE WINAPI GetLdrModule(HMODULE hMod) {
	PROCESS_BASIC_INFORMATION info;
	PLDR_MODULE mod;
	
	NtQueryInformationProcess(
	    GetCurrentProcess(), ProcessBasicInformation, &info, sizeof(info), NULL);

	mod = ((PPEB)info.PebBaseAddress)->LoaderData->InLoadOrderModuleList.Flink;
	while(mod) {
		if(mod->BaseAddress == hMod) return mod;
		mod = mod->InLoadOrderModuleList.Flink;
	}

	return NULL;
}

HMODULE WrapGetModule(HMODULE hMod) {
	if(GetModuleOriginal && hMod == thisModule) return hookedModule;
	return hMod;
}

HMODULE WINAPI XP_GetModuleHandleW(PWSTR lpModuleName) {
	return WrapGetModule(GetModuleHandleW(lpModuleName));
}

HMODULE WINAPI XP_GetModuleHandleA(LPCSTR lpModuleName) {
	return WrapGetModule(GetModuleHandleA(lpModuleName));
}

BOOL WINAPI XP_GetModuleHandleExW(DWORD dwFlags, PWSTR lpModuleName, HMODULE* phModule) {
	PROCESS_BASIC_INFORMATION info;
	PLDR_MODULE mod;

	if(dwFlags & GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS) {
		NtQueryInformationProcess(
		    GetCurrentProcess(), ProcessBasicInformation, &info, sizeof(info), NULL);
		mod = ((PPEB)info.PebBaseAddress)->LoaderData->InLoadOrderModuleList.Flink;
		while(mod) {
			if((DWORD)mod->BaseAddress <= (DWORD)lpModuleName && 
			   (DWORD)lpModuleName < (DWORD)mod->BaseAddress + (DWORD)mod->SizeOfImage) {
				*phModule = mod->BaseAddress;
				break;
			}
			mod = mod->InLoadOrderModuleList.Flink;
		}
	} else {
		*phModule = GetModuleHandleW(lpModuleName);
	}

	if(*phModule == NULL) return FALSE;
	*phModule = WrapGetModule(*phModule);

	mod = GetLdrModule(*phModule);
	if(!(dwFlags & GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT))
		mod->LoadCount++;

	//How to pin? We'll just inc the LoadCount and hope
	if(dwFlags & GET_MODULE_HANDLE_EX_FLAG_PIN) 
		mod->LoadCount++;
	
	return TRUE;
}

BOOL WINAPI XP_GetModuleHandleExA(DWORD dwFlags, LPCSTR lpModuleName, HMODULE* phModule) {
	BOOL ret;
	UNICODE_STRING unicode;
	STRING ansi;
	if(!(dwFlags & GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS)) {
		RtlInitString(&ansi, lpModuleName);
		RtlAnsiStringToUnicodeString(&unicode, &ansi, TRUE);
		ret = XP_GetModuleHandleExW(dwFlags, unicode.Buffer, phModule);
		RtlFreeUnicodeString(&unicode);
	} else {
		ret = XP_GetModuleHandleExW(dwFlags, lpModuleName, phModule);
	}	
	return ret;
}

DWORD WINAPI XP_GetProcessId(HANDLE Process) {
	PROCESS_BASIC_INFORMATION info;
	ULONG len;
	NTSTATUS status;
	
	status = NtQueryInformationProcess(
	    Process, ProcessBasicInformation, &info, sizeof(info), &len);

	if(NT_SUCCESS(status)) {
		return info.UniqueProcessId;
	}
	
	SetLastError(RtlNtStatusToDosError(status));
	return 0;
}

BOOL WINAPI XP_GetProcessHandleCount(HANDLE hProcess, PDWORD pdwHandleCount) {
	ULONG len;
	NTSTATUS status;
	
	status = NtQueryInformationProcess(
	    hProcess, ProcessHandleCount, pdwHandleCount, sizeof(DWORD), &len);
		
	if(NT_SUCCESS(status)) {
		return TRUE;
	}
	
	SetLastError(RtlNtStatusToDosError(status));
	return FALSE;
}

DWORD WINAPI XP_GetThreadId(HANDLE Thread) {
    THREAD_BASIC_INFORMATION tbi;
    ULONG len;
    NTSTATUS status;

    status = NtQueryInformationThread(
		Thread, ThreadBasicInformation, &tbi, sizeof(tbi), &len);

    if (NT_SUCCESS(status)) {
		return tbi.ClientId.UniqueThread;
    }

    SetLastError(RtlNtStatusToDosError(status));
    return 0;
}

void WINAPI XP_RestoreLastError(DWORD dwErrCode) {
	SetLastError(dwErrCode);	
}

/*
BOOL WINAPI XP_HeapSetInformation(
	HANDLE HeapHandle,
	HEAP_INFORMATION_CLASS HeapInformationClass,
	PVOID HeapInformation,
	SIZE_T HeapInformationLength) {
	//This function only supports HeapCompatibilityInformation on XP and
	//only is used to enable LFH.
	if(HeapInformationLength == sizeof(ULONG) && *(ULONG *)HeapInformation == 2) return TRUE;
	return FALSE;
}
*/

DWORD WINAPI XP_GetVersion(void) {
	return versionInfoA.dwMajorVersion | (versionInfoA.dwMinorVersion << 8) | (versionInfoA.dwBuildNumber << 16);
}

BOOL WINAPI XP_GetVersionExA(LPOSVERSIONINFOA lpVersionInfo) {
	if(lpVersionInfo->dwOSVersionInfoSize == sizeof(OSVERSIONINFOA) ||
		lpVersionInfo->dwOSVersionInfoSize == sizeof(OSVERSIONINFOEXA)) {
		memcpy(lpVersionInfo, &versionInfoA, lpVersionInfo->dwOSVersionInfoSize);
		return TRUE;
	}
	return GetVersionExA(lpVersionInfo);
}

BOOL WINAPI XP_GetVersionExW(LPOSVERSIONINFOW lpVersionInfo) {
	if(lpVersionInfo->dwOSVersionInfoSize == sizeof(OSVERSIONINFOW) ||
		lpVersionInfo->dwOSVersionInfoSize == sizeof(OSVERSIONINFOEXW)) {
		memcpy(lpVersionInfo, &versionInfoW, lpVersionInfo->dwOSVersionInfoSize);
		return TRUE;
	}
	return GetVersionExW(lpVersionInfo);
}

PVOID WINAPI XP_EncodePointer(PVOID ptr) {
	return ptr;
}

PVOID WINAPI XP_DecodePointer(PVOID ptr) {
	return ptr;
}

BYTE slist_lock[0x10000];

void SListLock(PSLIST_HEADER ListHead) {
	DWORD index = (((DWORD)ListHead) >> MEMORY_ALLOCATION_ALIGNMENT) & 0xFFFF;
	
	__asm {
		push ebx
		mov edx, index
		mov al, 0
		mov bl, 1
  spin:	lock cmpxchg byte ptr [slist_lock + edx], bl
		jnz spin
		pop ebx
	};

	return;
}

void SListUnlock(PSLIST_HEADER ListHead) {
	DWORD index = (((DWORD)ListHead) >> MEMORY_ALLOCATION_ALIGNMENT) & 0xFFFF;
	slist_lock[index] = 0;
}

//Note: ListHead->Next.Next == first node
PSLIST_ENTRY WINAPI XP_InterlockedPopEntrySList(PSLIST_HEADER ListHead) {
	PSLIST_ENTRY ret;
	SListLock(ListHead);
	if(!ListHead->Next.Next) ret = NULL;
	else { 
		ret = ListHead->Next.Next;
		ListHead->Next.Next = ret->Next;
		ListHead->Depth--;
		ListHead->Sequence++;
	}
	SListUnlock(ListHead);
	return ret;
}

PSLIST_ENTRY WINAPI XP_InterlockedPushEntrySList(PSLIST_HEADER ListHead, PSLIST_ENTRY ListEntry) {
	PSLIST_ENTRY ret;
	SListLock(ListHead);
	ret = ListHead->Next.Next;
	ListEntry->Next = ret;
	ListHead->Next.Next = ListEntry;
	ListHead->Depth++;
	ListHead->Sequence++;
	SListUnlock(ListHead);
	return ret;
}

void WINAPI XP_GetNativeSystemInfo(LPSYSTEM_INFO lpSystemInfo) {
	GetSystemInfo(lpSystemInfo);
	return;
}

DWORD WINAPI XP_WTSGetActiveConsoleSessionId(void) {
	DWORD SessionId;
	if(ProcessIdToSessionId(GetCurrentProcessId(), &SessionId)) return SessionId;
	else return 0xFFFFFFFF;
}

GEOID WINAPI XP_GetUserGeoID(GEOCLASS GeoClass) {
	switch(GeoClass) {
		case GEOCLASS_NATION: return geoNation;
		case GEOCLASS_REGION: return geoRegion;
		default: return GEOID_NOT_AVAILABLE;
	}
}

/* 47414b !   call        dword ptr [KERNEL32.dll:GetUserGeoID]
 474151 !   mov         edi, eax
 474153 !   cmp         edi, 0ffffffffh
 474156 !   jz          loc_4741d5
 474158 !   mov         esi, [KERNEL32.dll:GetGeoInfoA]
 47415e !   xor         eax, eax
 474160 !   push        eax ;LangId
 474161 !   push        eax ;cchData
 474162 !   push        eax ;lpGeoDate
 474163 !   push        4 ;GeoType=GEO_ISO2
 474165 !   push        edi ;Location*/

int WINAPI XP_GetGeoInfoA(GEOID Location, GEOTYPE GeoType, LPSTR lpGeoData, int cchData, LANGID LangId) {
	/*if(!LangId && (GeoType == GEO_LCID || GeoType == GEO_RFC1766)) {
		LangId = GetUserDefaultLangID();
	}*/
	switch(GeoType) {
		case GEO_ISO2:
			if(cchData == 0) return sizeof(geoISO2A);
			if(cchData < sizeof(geoISO2A)) return 0;
			memcpy(lpGeoData, geoISO2A, sizeof(geoISO2A));
			return sizeof(geoISO2A);
		case GEO_ISO3:
			if(cchData == 0) return sizeof(geoISO3A);
			if(cchData < sizeof(geoISO3A)) return 0;
			memcpy(lpGeoData, geoISO3A, sizeof(geoISO3A));
			return sizeof(geoISO3A);
		default: return 0;
	}
}

UINT WINAPI XP_GetSystemDirectoryW(PWSTR lpBuffer, UINT uSize) {
	int i;
	for(i = 0; systemDirectoryW[i]; i++) {
		if(i < uSize) lpBuffer[i] = systemDirectoryW[i];
	}
	if(i < uSize) { lpBuffer[i] = L'\0'; return i; }
	return i + 1;
}

UINT WINAPI XP_GetSystemDirectoryA(LPSTR lpBuffer, UINT uSize) {
	int i;
	for(i = 0; systemDirectoryA[i]; i++) {
		if(i < uSize) lpBuffer[i] = systemDirectoryA[i];
	}
	if(i < uSize) { lpBuffer[i] = '\0'; return i; }
	return i + 1;
}

BOOL WINAPI XP_GetNumaNodeProcessorMask(UCHAR Node, PULONGLONG ProcessorMask) {
	DWORD_PTR ProcessAffinity;
	DWORD_PTR SystemAffinity;
	
	if(GetProcessAffinityMask(GetCurrentProcess(), &ProcessAffinity, &SystemAffinity)) {
		*ProcessorMask = SystemAffinity;
		return TRUE;
	} else {
		return FALSE;
	}
}

BOOL WINAPI XP_GetNumaProcessorNode(UCHAR Processor, PUCHAR NodeNumber) {
	DWORD_PTR ProcessAffinity;
	DWORD_PTR SystemAffinity;
	
	if(GetProcessAffinityMask(GetCurrentProcess(), &ProcessAffinity, &SystemAffinity)) {
		if(1<<Processor & SystemAffinity) *NodeNumber = 0;
		else *NodeNumber = 0xFF;
		return TRUE;
	} else {
		return FALSE;
	}
}

/*
BOOL WINAPI XP_GetVolumePathNamesForVolumeNameA(
  LPCSTR lpszVolumeName, LPSTR lpszVolumePathNames, DWORD cchBufferLength, PDWORD lpcchReturnLength) {
	CHAR buffer[MAX_PATH+1];
	HANDLE findVMP;
	findVMP = FindFirstVolumeMountPointA(lpszVolumeName, buffer, sizeof(buffer));
	if(findVMP == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	lstrcpynA(lpszVolumePathNames, buffer, cchBufferLength);
}

BOOL WINAPI XP_GetVolumePathNamesForVolumeNameW(
	LPCWSTR lpszVolumeName, LPWSTR lpszVolumePathNames, DWORD cchBufferLength, PDWORD lpcchReturnLength) {
}*/

HMODULE WINAPI XP_LoadLibraryA(LPCSTR lpFileName) {
	return LoadLibraryA(lpFileName);
}

HMODULE WINAPI XP_LoadLibraryW(LPCWSTR lpFileName) {
	return LoadLibraryW(lpFileName);
}

FARPROC WINAPI XP_GetProcAddress(HMODULE hModule, LPCSTR lpProcName) {
	FARPROC p;
	if(RewriteExportTable && hModule == thisModule) {
		 p = GetProcAddress(hookedModule, lpProcName);
		 if(p) return p;
	}
	return GetProcAddress(hModule, lpProcName);
}

BOOL WINAPI XP_CheckRemoteDebuggerPresent(HANDLE hProcess, PBOOL pbDebuggerPresent) {
	*pbDebuggerPresent = FALSE;
	return FALSE;
}

#define EVALRVA(type, x) ((type)((DWORD)hModule + (DWORD)(x)))
PIMAGE_DATA_DIRECTORY GetDataDirectory(HMODULE hModule, int directory) {
	PIMAGE_DOS_HEADER dosheader;
	PIMAGE_NT_HEADERS ntheader;
	dosheader = EVALRVA(PIMAGE_DOS_HEADER, 0);
	ntheader  = EVALRVA(PIMAGE_NT_HEADERS, dosheader->e_lfanew);
	return &ntheader->OptionalHeader.DataDirectory[directory];
}

void RenameExportTableName(HMODULE hModule, const char *name) {
	PIMAGE_DATA_DIRECTORY exportdd;
	PIMAGE_EXPORT_DIRECTORY exportd;
	DWORD oldProtect;
	
	exportdd  = GetDataDirectory(hModule, IMAGE_DIRECTORY_ENTRY_EXPORT);
	exportd   = EVALRVA(PIMAGE_EXPORT_DIRECTORY, exportdd->VirtualAddress);
	
	VirtualProtect(EVALRVA(DWORD *, exportd->Name), sizeof(DWORD), PAGE_EXECUTE_READWRITE, &oldProtect);
	*EVALRVA(DWORD *, exportd->Name) = (DWORD)name - (DWORD)hModule; //RVA from module
	VirtualProtect(EVALRVA(DWORD *, exportd->Name), sizeof(DWORD), oldProtect, &oldProtect);
}

void ForwardExportsToFunctionExports(HMODULE hModule) {
	PIMAGE_DATA_DIRECTORY exportdd;
	PIMAGE_EXPORT_DIRECTORY exportd;
	DWORD *func_addresses;
	DWORD func_address;
	DWORD oldProtect;
	int i;
	
	exportdd  = GetDataDirectory(hModule, IMAGE_DIRECTORY_ENTRY_EXPORT);
	exportd   = EVALRVA(PIMAGE_EXPORT_DIRECTORY, exportdd->VirtualAddress);
	func_addresses = EVALRVA(DWORD *, exportd->AddressOfFunctions);
	
	VirtualProtect(func_addresses, sizeof(DWORD) * exportd->NumberOfFunctions, PAGE_EXECUTE_READWRITE, &oldProtect);
	for(i = 0; i < exportd->NumberOfFunctions; i++) {
		if(exportdd->VirtualAddress <= func_addresses[i] &&
		  func_addresses[i] < exportdd->VirtualAddress + exportdd->Size) {
			func_address = GetProcAddress(hModule, i);
			if(func_address)
				func_addresses[i - exportd->Base] = func_address - (DWORD)hModule; //RVA from module
		}
	}
	VirtualProtect(func_addresses, sizeof(DWORD) * exportd->NumberOfFunctions, oldProtect, &oldProtect);
}
#undef EVALRVA

BOOL WINAPI XP_GetSystemTimes(LPFILETIME lpIdleTime, LPFILETIME lpKernelTime, LPFILETIME lpUserTime) {
	SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION info;
	NTSTATUS status;
	ULONG len;
	status = NtQuerySystemInformation(
		SystemProcessorPerformanceInformation, &info, sizeof(info), &len);
	if(NT_SUCCESS(status)) {
		*lpKernelTime = *(PFILETIME)&info.KernelTime;
		*lpUserTime   = *(PFILETIME)&info.UserTime;
		*lpIdleTime   = *(PFILETIME)&info.IdleTime;
		return TRUE;
	} else {
		SetLastError(RtlNtStatusToDosError(status));
		return FALSE;
	}
}

UINT WINAPI XP_GetSystemWow64DirectoryA(LPSTR lpBuffer, UINT uSize) {
	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
	return 0;
}

UINT WINAPI XP_GetSystemWow64DirectoryW(LPWSTR lpBuffer, UINT uSize) {
	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
	return 0;
}

BOOL WINAPI DllMain(
  HINSTANCE hinstDLL,
  DWORD fdwReason,
  LPVOID lpvReserved) {
	static char buffer[MAX_PATH];
	static char ini_filename[] = "\\win2k_xp_kernel32.ini";
	DWORD ver, read;
	int i;
	
	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			thisModule   = hinstDLL;
			hookedModule = GetModuleHandleA("KERNEL32_ORG.DLL");
			
			versionInfoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
			versionInfoW.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXW);
			GetVersionExA(&versionInfoA);
			GetVersionExW(&versionInfoW);
				
			GetSystemDirectoryW(systemDirectoryW, sizeof(systemDirectoryW));
			GetSystemDirectoryA(systemDirectoryA, sizeof(systemDirectoryA));

			read = GetCurrentDirectoryA(sizeof(buffer), buffer);
			if(read == 0 || read + sizeof(ini_filename) >= sizeof(buffer)) break;
			lstrcatA(buffer, ini_filename);

			versionInfoA.dwMajorVersion = versionInfoW.dwMajorVersion = 
				GetPrivateProfileIntA("win2k_xp_kernel32", "majVer", versionInfoA.dwMajorVersion, buffer);
			versionInfoA.dwMinorVersion = versionInfoW.dwMinorVersion = 
				GetPrivateProfileIntA("win2k_xp_kernel32", "minVer", versionInfoA.dwMinorVersion, buffer);
			versionInfoA.dwBuildNumber = versionInfoW.dwBuildNumber = 
				GetPrivateProfileIntA("win2k_xp_kernel32", "buildNum", versionInfoA.dwBuildNumber, buffer);
				
			versionInfoA.wServicePackMajor = versionInfoW.wServicePackMajor = 
				GetPrivateProfileIntA("win2k_xp_kernel32", "majSP", versionInfoA.wServicePackMajor, buffer);
			versionInfoA.wServicePackMinor = versionInfoW.wServicePackMinor = 
				GetPrivateProfileIntA("win2k_xp_kernel32", "minSP", versionInfoA.wServicePackMinor, buffer);
				
			GetPrivateProfileStringA("win2k_xp_kernel32", "versionString", versionInfoA.szCSDVersion, versionInfoA.szCSDVersion, sizeof(versionInfoA.szCSDVersion), buffer);
			MultiByteToWideChar(CP_ACP, 0, versionInfoA.szCSDVersion, -1, versionInfoW.szCSDVersion, sizeof(versionInfoW.szCSDVersion));
			
			GetModuleOriginal = GetPrivateProfileIntA("win2k_xp_kernel32", "GetModuleOriginal", 0, buffer);
			
			//Default to United States
			geoNation = GetPrivateProfileIntA("win2k_xp_kernel32", "geoNation", 0xF4, buffer);
			geoRegion = GetPrivateProfileIntA("win2k_xp_kernel32", "geoRegion", 0xF4, buffer);
			GetPrivateProfileStringA("win2k_xp_kernel32", "geoISO2", "US",  geoISO2A, sizeof(geoISO2A), buffer);
			GetPrivateProfileStringA("win2k_xp_kernel32", "geoISO3", "USA", geoISO3A, sizeof(geoISO3A), buffer);
			
			GetPrivateProfileStringA("win2k_xp_kernel32", "SystemDirectory", NULL, systemDirectoryA, sizeof(systemDirectoryA), buffer);			
			if(systemDirectoryA[0] ==  '\0') GetSystemDirectoryA(systemDirectoryA, sizeof(systemDirectoryA));
			MultiByteToWideChar(CP_ACP, 0, systemDirectoryA, -1, systemDirectoryW, sizeof(systemDirectoryW));
			
			if(GetPrivateProfileIntA("win2k_xp_kernel32", "Load_ACLayers", 0, buffer)) {
				//oh noes I'm calling loadlibrary in dllmain
				LoadLibraryA("aclayers.dll");
			}
			
			RewriteExportTable = GetPrivateProfileIntA("win2k_xp_kernel32", "RewriteExportTable", 0, buffer);
			if(RewriteExportTable) {
				RenameExportTableName(hookedModule, "KERNEL32_ORG.dll");
				ForwardExportsToFunctionExports(thisModule);
			}
			
			break;
	}

	return TRUE;
}

